@@ -35,6 +35,7 @@
 
 #include <private/qqmlmetatype_p.h>
 #include <private/qqmlpropertyvalueinterceptor_p.h>
+#include <private/qqmlengine_p.h>
 #include <private/qhashedstring_p.h>
 #include "../../shared/util.h"
 
@@ -64,6 +65,7 @@ private slots:
     void unregisterCustomSingletonType();
 
     void normalizeUrls();
+    void unregisterAttachedProperties();
 };
 
 class TestType : public QObject
@@ -227,17 +229,13 @@ void tst_qqmlmetatype::qmlType()
 
 void tst_qqmlmetatype::invalidQmlTypeName()
 {
-    QStringList currFailures = QQmlMetaType::typeRegistrationFailures();
+    QTest::ignoreMessage(QtWarningMsg, "Invalid QML element name \"testtype\"; type names must begin with an uppercase letter");
+    QTest::ignoreMessage(QtWarningMsg, "Invalid QML element name \"Test$Type\"");
+    QTest::ignoreMessage(QtWarningMsg, "Invalid QML element name \"EndingInSlash/\"");
+
     QCOMPARE(qmlRegisterType<TestType>("TestNamespace", 1, 0, "Test$Type"), -1); // should fail due to invalid QML type name.
     QCOMPARE(qmlRegisterType<TestType>("Test", 1, 0, "EndingInSlash/"), -1);
-    QStringList nowFailures = QQmlMetaType::typeRegistrationFailures();
-
-    foreach (const QString &f, currFailures)
-        nowFailures.removeOne(f);
-
-    QCOMPARE(nowFailures.size(), 2);
-    QCOMPARE(nowFailures.at(0), QStringLiteral("Invalid QML element name \"Test$Type\""));
-    QCOMPARE(nowFailures.at(1), QStringLiteral("Invalid QML element name \"EndingInSlash/\""));
+    QCOMPARE(qmlRegisterType<TestType>("Test", 1, 0, "testtype"), -1);
 }
 
 void tst_qqmlmetatype::prettyTypeName()
@@ -537,6 +535,43 @@ void tst_qqmlmetatype::normalizeUrls()
     QVERIFY(!QQmlMetaType::qmlType(url, /*includeNonFileImports=*/true).isValid());
 }
 
+void tst_qqmlmetatype::unregisterAttachedProperties()
+{
+    qmlClearTypeRegistrations();
+
+    const QUrl dummy("qrc:///doesnotexist.qml");
+    {
+        QQmlEngine e;
+        QQmlComponent c(&e);
+        c.setData("import QtQuick 2.2\n Item { }", dummy);
+
+        const QQmlType attachedType = QQmlMetaType::qmlType("QtQuick/KeyNavigation", 2, 2);
+        QCOMPARE(attachedType.attachedPropertiesId(QQmlEnginePrivate::get(&e)),
+                 attachedType.index());
+
+        QVERIFY(c.create());
+    }
+
+    qmlClearTypeRegistrations();
+    {
+        QQmlEngine e;
+        QQmlComponent c(&e);
+
+        // The extra import shuffles the type IDs around, so that we
+        // get a different ID for the attached properties. If the attached
+        // properties aren't properly cleared, this will crash.
+        c.setData("import QtQml.StateMachine 1.0 \n"
+                  "import QtQuick 2.2 \n"
+                  "Item { KeyNavigation.up: null }", dummy);
+
+        const QQmlType attachedType = QQmlMetaType::qmlType("QtQuick/KeyNavigation", 2, 2);
+        QCOMPARE(attachedType.attachedPropertiesId(QQmlEnginePrivate::get(&e)),
+                 attachedType.index());
+
+        QVERIFY(c.create());
+    }
+}
+
 QTEST_MAIN(tst_qqmlmetatype)
 
 #include "tst_qqmlmetatype.moc"
